# btsutils -- Python module to interact with debbugs servers.
# Copyright (C) 2007 Gustavo R. Montesino
#
# Some parts of this code are based on reportbug-ng from Bastian Venthur
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import urllib
import re

from bug import Bug

html_url = "http://bugs.debian.org/"

def connect(url="http://bugs.debian.org/"):
	"""Change the debbugs server to use"""

	html_url = url
	
def get(bugnumber):
	"""Returns a single bug through HTML parsing"""

	socket = urllib.urlopen(html_url + str(bugnumber))
	html = socket.read()

	match = re.findall("<BODY>(.*)</h3>", html, re.DOTALL)
	if match:
		block = match[0]

		package = re.findall("^Package: <a.*>(.*)</a>;$", block, re.MULTILINE)
		summary = re.findall("^<H1>Debian Bug report logs - .*<BR>(.*)</H1>$", block, re.MULTILINE)
		severity = re.findall("^<h3>Severity: (.*);$", block, re.MULTILINE)
		submitter = re.findall("^<br>Reported by: <a.*>(.*)</a>;$", block, re.MULTILINE)
		tags = re.findall("^Tags: (.*);$", block, re.MULTILINE)

		report = Bug(bug=bugnumber)
		report.setPackage(decode_entities(package[0]))
		report.setSummary(decode_entities(summary[0]))

		if severity:
			report.setSeverity(decode_entities(severity[0]))
		else:
			report.setSeverity("normal")

		report.setSubmitter(decode_entities(submitter[0]))

		if tags:
			report.setTags(tags[0].split(", "))
		else:
			report.setTags([])

		return report
	else:
		return None

def queryToURL(query):
	"""Transforms a query string into a debbugs URL"""

	queryurl = html_url + "cgi-bin/pkgreport.cgi?";

	# Source package search
	if query.startswith("src:"):
		queryurl = "%swhich=src&data=%s" % (queryurl, query[4:].strip())

	# Binary package search
	elif query.startswith("pkg:"):
		queryurl = "%swhich=pkg&data=%s" % (queryurl, query[4:].strip());

	# Maintainer e-mail address search
	elif query.startswith("maint:"):
		queryurl = "%swhich=maint&data=%s" % (queryurl, query[6:].strip());

	# Submitter e-mail address search
	elif query.startswith("from:"):
		queryurl = "%swhich=submitter&data=%s" % (queryurl, query[5:].strip());

	else:
		raise SyntaxError, query

	# For now, get only pending bugs
	queryurl = "%s&pend-inc=pending" % (queryurl)

	return queryurl;
	
def query(query):
	"""Returns a list with the bugs which match query."""

	if query.startswith("bug:"):
		return [ get(query[4:].strip()) ];

	cluster_re = re.compile("(<H2.*?><a.*?></a>.* bugs -- .*? .*</H2>)")
	status_severity_re = re.compile("<H2.*?><a.*?></a>(.*) bugs -- (.*?) .*</H2>")
	number_summary_package_re = re.compile("""^<li><a href=\"bugreport.cgi\?bug=[0-9]*\">#([0-9]*): (.*)</a>$
^<br>Package: <a class=\"submitter\" href=\"pkgreport.cgi\?pkg=.*?\">(.*?)</a>.*;$
^Severity: <em>(.*)</em>;$
^Reported by: <a.*>(.*)</a>;$
(?:^Tags: <strong>(.*)</strong>;$)?""", re.MULTILINE)

	report = urllib.urlopen(queryToURL(query));
	s = report.read()

	bugs = []

	cluster = cluster_re.split(s)

	last = None
	for i in cluster:
		match = number_summary_package_re.findall(i)
		if match:
			(status, severity) = status_severity_re.findall(last)[0]
			for j in match:
				bug = Bug(bug=j[0])
				bug.setSummary(decode_entities(j[1]))
				bug.setPackage(decode_entities(j[2]))
				bug.setSeverity(decode_entities(j[3]))
				bug.setSubmitter(decode_entities(j[4]))
				bug.setTags(j[5].split(", "))

				bugs.append(bug)
		last = i

	return bugs

def decode_entities(htmlstring):
	"""Decodes html entities from string"""

	# Debbugs uses HTML::Entites perl module for encoding "dangerous
	# characters"; so we have to decode them. According to perl docs,
	# The encoded entities which interests us are <, &, >, ' and "

	htmlstring = htmlstring.replace("&lt;", "<")
	htmlstring = htmlstring.replace("&amp;", "&")
	htmlstring = htmlstring.replace("&gt;", ">")
	htmlstring = htmlstring.replace("&quot;", "\"")
	htmlstring = htmlstring.replace("&#39;", "'")

	return htmlstring
