# btsutils -- Python module to interact with debbugs servers.
# Copyright (C) 2007  Gustavo R. Montesino
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import SOAPpy

from bug import Bug

soap_url = "bugs.donarmstrong.com/cgi-bin/soap.cgi"
soap_namespace = "Debbugs/SOAP/Status"
soap_proxy = SOAPpy.SOAPProxy(soap_url, soap_namespace)

def connect(url="bugs.donarmstrong.com/cgi-bin/soap.cgi", namespace="Debbugs/SOAP/Status"):
	"""Change the debbugs server to use"""

	soap_url = url
	soap_namespace = namespace
	soap_proxy = SOAPpy.SOAPProxy(url, namespace)
	
def get(bugnumber):
	"""Return a single bug through SOAP
	
	Query the SOAP server and return an Bug instance
	with the bug data.
	"""

	soap_proxy.soapaction = "%s#get_status" % soap_namespace
	raw = soap_proxy.get_status(bugnumber)['item']['value']

	report = Bug()
	report.setBug(bugnumber)
	report.setPackage(raw['package'])
	report.setSummary(raw['subject'])
	report.setSeverity(raw['severity'])
	report.setSubmitter(raw['originator'])
	report.setTags(raw['tags'].split())

	return report
